/*****************************************************************************
 * Copyright (C) 2010 S.R.Haque <srhaque@theiet.org>                         *
 * Christoph Thielecke <crissi99@gmx.de> (broken out from kvpnc.h)           *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This package is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this package; if not, write to the Free Software               *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA *
 *****************************************************************************/

#include <QtCore/QString>
#include "HandlerBase.h"

class KProcess;

class RacoonHandler:
        public HandlerBase
{
    Q_OBJECT
public:
    RacoonHandler(
        class KVpnc *ui,
        class KVpncConfig *config,
        QString type,
        QString daemons,
        bool _useL2tpAfterFirstConnection);
    bool vpnStart(bool authenticationRetry);
    void vpnStop();
    QString pingDevice() const;

private:
    HelperProcess *RacoonProcess; //< racoon
    HelperProcess *RacoonHelperProcess;
    HelperProcess *RacoonTailProcess;
    HelperProcess *RacoonctlProcess; //< racoonctl
    int RacoonctlConnectCounter; //<< counter for waiting of adminsock
    int racoon_tunnel_state; // 0 => no tunnel, 1 => first tunnel, 2 => second tunnel
    QString tmpInterface;
    QString L2tpPppDevice;

    /**
     * TBD: can this be shared by other handlers?
     */
    bool getAuthenticationData(bool AuthRetry);

private slots:
    void racoonOutputReader();
    void racoonErrorReader();
    void racoonParser(QString msg, const QString &source);
    void readFromStdout_racoon_helper();
    void readFromStderr_racoon_helper();
    void readFromStdout_tail_racoon();
    void readFromStderr_tail_racoon();
    void readFromStdout_racoonctl();
    void readFromStderr_racoonctl();
    void racoonctlExited(int, QProcess::ExitStatus);
    void doRacoon(int, QProcess::ExitStatus);
    void stopRacoon();
    void doTailRacoonLog();
    void doIptablesRacoon();
    void addRouteRacoon();
    void delRouteRacoon();
    void removeIptablesRacoon();
    void racoonStarted();
    void removeSetkey();
};
