/*
 * This file is part of people
 *
 * Copyright (C) 2010 Collabora Ltd. <info@collabora.co.uk>
 *   @author George Goldberg <george.goldberg@collabora.co.uk>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "people-manager.h"

#include <KDebug>

class PeopleManager::Private {

public:
    Private()
    { }

    static QList<QWeakPointer<AbstractPeopleManagerInterface> > interfaceCache;
};

QList<QWeakPointer<AbstractPeopleManagerInterface> > PeopleManager::Private::interfaceCache;

PeopleManager::PeopleManager(QObject *parent)
  : QObject(parent),
    d(new PeopleManager::Private)
{
    kDebug();
}

PeopleManager::~PeopleManager()
{
    kDebug();

    delete d;
}

QList<QWeakPointer<AbstractPeopleManagerInterface> > PeopleManager::cachedInterfaces() const
{
    kDebug();

    return d->interfaceCache;
}

void PeopleManager::cacheInterface(QSharedPointer<AbstractPeopleManagerInterface> interface)
{
    // FIXME: Should we garbage collect the interface cache here?
    d->interfaceCache.append(interface.toWeakRef());
}


#include "people-manager.moc"

