/*
 * Copyright 2010  Jonas Emanuel Müller <zanoi@zanoi.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "channel.h"

namespace Videocatcher {

struct Channel::ChannelPrivate
{
    QString channelTitle;
    QString channelUrl;
    QString channelDescription;
    QString channelAuthor;
    QString channelWebsiteUrl;
    QList<Episode> channelEpisodes;
};

Channel::Channel() : d(new ChannelPrivate)
{
}

Channel::~Channel()
{
    delete d;
}


QString Channel::title() const
{
    return d->channelTitle;
}

QString Channel::url() const
{
    return d->channelUrl;
}

QString Channel::description() const
{
    return d->channelDescription;
}

QString Channel::author() const
{
    return d->channelAuthor;
}

QString Channel::websiteUrl() const
{
    return d->channelWebsiteUrl;
}

QList<Episode> Channel::episodes() const
{
    return d->channelEpisodes;
}

void Channel::setTitle(const QString &title)
{
    d->channelTitle = title;
}

void Channel::setUrl(const QString &url)
{
    d->channelUrl = url;
}

void Channel::setDescription(const QString &description)
{
    d->channelDescription = description;
}

void Channel::setAuthor(const QString &author)
{
    d->channelAuthor = author;
}

void Channel::setWebsiteUrl(const QString &websiteUrl)
{
    d->channelWebsiteUrl = websiteUrl;
}

void Channel::setEpisodes(const QList<Episode> &episodes)
{
    d->channelEpisodes = episodes;
}

} //namespace Videocatcher
